@@ -63,9 +63,10 @@ extern short aet_sizes[];
 #define N_FIXNUM_ARGS 6
 
 DEFUNO_NEW("AREF", object, fLaref, LISP, 1, ARRAY_RANK_LIMIT,
-       NONE, OO, II, II, II,void,Laref,(object x,fixnum i, ...),"")
+       NONE, OO, OO, OO, OO,void,Laref,(object x,object oi, ...),"")
 { int n = VFUN_NARGS;
   int i1;
+  fixnum i=n>1 ? fix(oi) : 0;
   va_list ap;
   if (type_of(x) == t_array)
     {int m ;
@@ -75,7 +76,7 @@ DEFUNO_NEW("AREF", object, fLaref, LISP, 1, ARRAY_RANK_LIMIT,
        FEerror(" ~a has wrong rank",1,x);
      if (rank == 1) return fLrow_major_aref(x,i);
      if (rank == 0) return fLrow_major_aref(x,0);
-     va_start(ap,i);
+     va_start(ap,oi);
      m = 0;
      k = i;
      /* index into 1 dimensional array */
@@ -90,7 +91,7 @@ DEFUNO_NEW("AREF", object, fLaref, LISP, 1, ARRAY_RANK_LIMIT,
 	 if (m <= rank)
 	   { i1 = i1 * x->a.a_dims[m];
 	     if (m < N_FIXNUM_ARGS)
-	       { k = va_arg(ap,fixnum);}
+	       { k = fixint(va_arg(ap,object));}
 	     else {object x = va_arg(ap,object);
 		   check_type(x,t_fixnum);
 		   k = Mfix(x);}
@@ -210,14 +211,18 @@ DEFUN_NEW("ASET1", object, fSaset1, SI, 3, 3, NONE, OO, IO, OO,OO,(object x, fix
       break;
     case aet_bit:
       i +=  BV_OFFSET(x);
-    AGAIN_BIT: 
       ASSURE_TYPE(val,t_fixnum);
-      {int v = Mfix(val);
-       if (v == 0) CLEAR_BITREF(x,i);
-       else if (v == 1) SET_BITREF(x,i);
-       else {val= fSincorrect_type(val,sLbit);
-	     goto AGAIN_BIT;}
-       break;}
+      switch (Mfix(val)) {
+      case 0:
+	CLEAR_BITREF(x,i);
+	break;
+      case 1:
+	SET_BITREF(x,i);
+	break;
+      default:
+	TYPE_ERROR(val,sLbit);
+      }
+      break;
     case aet_fix:
       ASSURE_TYPE(val,t_fixnum);
       (x->fixa.fixa_self[i]) = Mfix(val);
@@ -348,12 +353,13 @@ DEFUNO_NEW("SVSET", object, fSsvset, SI, 3, 3, NONE, OO, IO, OO,
 */ 
 
 
-DEFUN_NEW("MAKE-VECTOR1",object,fSmake_vector1,SI,3,8,NONE,OI,
-      IO,OO,OO,(fixnum n,fixnum elt_type,object staticp,...),"")
+DEFUN_NEW("MAKE-VECTOR1",object,fSmake_vector1,SI,3,8,NONE,OO,
+      OO,OO,OO,(object on,object oelt_type,object staticp,...),"")
   
 { 
     int displaced_index_offset=0;
     int Inargs = VFUN_NARGS - 3;
+    fixnum n=fixint(on),elt_type=fixint(oelt_type);
     va_list Iap;object fillp;object initial_element;object displaced_to;object V9;
     Inargs = VFUN_NARGS - 3 ;
     { object x;
@@ -441,7 +447,12 @@ DEFUN_NEW("MAKE-VECTOR1",object,fSmake_vector1,SI,3,8,NONE,OI,
 object 
 fSmake_vector1_1(fixnum n,fixnum elt_type,object staticp) {
   VFUN_NARGS=3;
-  return FFN(fSmake_vector1)(n,elt_type,staticp);
+  return FFN(fSmake_vector1)(make_fixnum(n),make_fixnum(elt_type),staticp);
+}
+object 
+fSmake_vector1_2(fixnum n,fixnum elt_type,object staticp,object fillp) {
+  VFUN_NARGS=4;
+  return FFN(fSmake_vector1)(make_fixnum(n),make_fixnum(elt_type),staticp,fillp);
 }
 
 
@@ -455,15 +466,15 @@ static longfloat DFLT_aet_lf = 0.0;
 static object Iname_t = sLt;
 static struct { char * dflt; object *namep;} aet_types[] =
 {   {(char *)	&DFLT_aet_object,	&Iname_t,},	/*  t  */
-    {(char *)	&DFLT_aet_ch, &sLstring_char,},/*  string-char  */
+    {(char *)	&DFLT_aet_ch, &sLcharacter,},/*  character  */
     {(char *)	&DFLT_aet_fix, &sLbit,},		/*  bit  */
     {(char *)	&DFLT_aet_fix,	&sLfixnum,}, 	/*  fixnum  */
     {(char *)	&DFLT_aet_sf, &sLshort_float,},			/*  short-float  */
     {(char *)	&DFLT_aet_lf, &sLlong_float,},	/*  long-float  */
-    {(char *)	&DFLT_aet_char,&sLsigned_char,},               /* signed char */
-    {(char *)    &DFLT_aet_char,&sLunsigned_char,},               /* unsigned char */
-    {(char *)	&DFLT_aet_short,&sLsigned_short,},              /* signed short */
-    {(char *)	&DFLT_aet_short, &sLunsigned_short},    /*  unsigned short   */
+    {(char *)	&DFLT_aet_char,&sSsigned_char,},               /* signed char */
+    {(char *)   &DFLT_aet_char,&sSunsigned_char,},               /* unsigned char */
+    {(char *)	&DFLT_aet_short,&sSsigned_short,},              /* signed short */
+    {(char *)	&DFLT_aet_short, &sSunsigned_short},    /*  unsigned short   */
 	};
 
 DEFUN_NEW("GET-AELTTYPE",object,fSget_aelttype,SI,1,1,NONE,OO,OO,OO,OO,(object x),"")
@@ -506,8 +517,8 @@ DEFUNO_NEW("MAKE-VECTOR",object,fSmake_vector,SI,7,8,NONE,
   /* 8 args */
 
   VFUN_NARGS = 8;
-  x = FFN(fSmake_vector1)(Mfix(x1),  /* n */
-		     fix(fSget_aelttype(x0)), /*aelt type */
+  x = FFN(fSmake_vector1)(x1,  /* n */
+		     fSget_aelttype(x0), /*aelt type */
 		     x6, /* staticp */
 		     x3, /* fillp */ 
 		     initial_elt, /* initial element */
@@ -548,6 +559,8 @@ DEFUN_NEW("MAKE-ARRAY1",object,fSmake_array1,SI,6,6,
       { x->a.a_dims[i] = FIX_CHECK(Mcar(v));
 	if (x->a.a_dims[i] < 0)
 	  { FEerror("Dimension must be non negative",0);}
+	if (dim && x->a.a_dims[i]>((1UL<<(sizeof(dim)*8-1))-1)/dim)
+	  FEerror("Total dimension overflow on dimensions ~s",1,dimensions);
 	dim *= x->a.a_dims[i++];
 	v = Mcdr(v);}
     x->a.a_dim = dim;
@@ -757,8 +770,8 @@ void
 adjust_displaced(object x, long diff)
 {
 	if (x->ust.ust_self != NULL)
-		x->ust.ust_self = (char *)((long)(x->a.a_self) + diff);
-	for (x = Mcdr(x->ust.ust_displaced);  x != Cnil;  x = Mcdr(x))
+		x->ust.ust_self = (unsigned char *)((long)(x->a.a_self) + diff);
+	for (x = Scdr(x->ust.ust_displaced);  x != Cnil;  x = Scdr(x))
 		adjust_displaced(Mcar(x), diff);
 }
 
@@ -841,31 +854,33 @@ raw_aet_ptr(object x, short int typ)
 	*/     
 
 void
-gset(void *p1, void *val, int n, int typ)
-{ if (val==0)
+gset(void *p1, void *val, int n, int typ) {
+
+  if (val==0)
     val = aet_types[typ].dflt;
-    switch (typ){
+
+  switch (typ){
 
 #define GSET(p,n,typ,val) {typ x = *((typ *) val); GSET1(p,n,typ,x)}
-#define GSET1(p,n,typ,val) while (n-- > 0) \
+#define GSET1(p,n,typ,val) while (n-- > 0)	\
       { *((typ *) p) = val; \
-	  p = p + sizeof(typ); \
-	  } break;
+	p = p + sizeof(typ);			\
+      } break;
 
-    case aet_object: GSET(p1,n,object,val);
-    case aet_ch:     GSET(p1,n,char,val);
-      /* Note n is number of fixnum WORDS for bit */
-    case aet_bit:    GSET(p1,n,fixnum,val);
-    case aet_fix:    GSET(p1,n,fixnum,val);
-    case aet_sf:     GSET(p1,n,shortfloat,val);
-    case aet_lf:     GSET(p1,n,longfloat,val);
-    case aet_char:   GSET(p1,n,char,val);
-    case aet_uchar:  GSET(p1,n,unsigned char,val);
-    case aet_short:  GSET(p1,n,short,val);
-    case aet_ushort: GSET(p1,n,unsigned short,val);
-    default:         FEerror("bad elttype",0);
-    }
+  case aet_object: GSET(p1,n,object,val);
+  case aet_ch:     GSET(p1,n,char,val);
+    /* Note n is number of fixnum WORDS for bit */
+  case aet_bit:    GSET(p1,n,fixnum,val);
+  case aet_fix:    GSET(p1,n,fixnum,val);
+  case aet_sf:     GSET(p1,n,shortfloat,val);
+  case aet_lf:     GSET(p1,n,longfloat,val);
+  case aet_char:   GSET(p1,n,char,val);
+  case aet_uchar:  GSET(p1,n,unsigned char,val);
+  case aet_short:  GSET(p1,n,short,val);
+  case aet_ushort: GSET(p1,n,unsigned short,val);
+  default:         FEerror("bad elttype",0);
   }
+}
 
 
 #define W_SIZE (BV_BITS*sizeof(fixnum))    
@@ -874,48 +889,54 @@ gset(void *p1, void *val, int n, int typ)
    */
 
 DEFUN_NEW("COPY-ARRAY-PORTION",object,fScopy_array_portion,SI,4,
-      5,NONE,OO,OI,IO,OO,(object x,object y,fixnum i1,fixnum i2,object n1o),
+      5,NONE,OO,OO,OO,OO,(object x,object y,object oi1,object oi2,object n1o),
    "Copy elements from X to Y starting at x[i1] to x[i2] and doing N1 \
 elements if N1 is supplied otherwise, doing the length of X - I1 \
 elements.  If the types of the arrays are not the same, this has \
 implementation dependent results.")
-{ enum aelttype typ1=Iarray_element_type(x);
+{ fixnum i1=fix(oi1),i2=fix(oi2);
+  enum aelttype typ1=Iarray_element_type(x);
   enum aelttype typ2=Iarray_element_type(y);
   int n1=fix(n1o),nc;
   if (VFUN_NARGS==4)
     { n1 = x->v.v_dim - i1;}
-  if (typ1==aet_bit)
-    {if (i1 % CHAR_SIZE)
-     badcopy:
-       FEerror("Bit copies only if aligned",0);
-    else
-      {int rest=n1%CHAR_SIZE;
-       if (rest!=0 )
-	 {if (typ2!=aet_bit)
-	    goto badcopy;
-	    {while(rest> 0)
-	       { fSaset1(y,i2+n1-rest,(fLrow_major_aref(x,i1+n1-rest)));
-		 rest--;}
-	     }}
-       i1=i1/CHAR_SIZE ;
-       n1=n1/CHAR_SIZE;
-       typ1=aet_char;
-     }};
-  if (typ2==aet_bit)
-    {if (i2 % CHAR_SIZE)
-       goto badcopy;
-       i2=i2/CHAR_SIZE ;}
-  if ((typ1 ==aet_object ||
-       typ2  ==aet_object) && typ1 != typ2)
+  if (typ1==aet_bit) {
+    if (i1 % CHAR_SIZE)
+    badcopy:
+      FEerror("Bit copies only if aligned",0);
+    else {
+      int rest=n1%CHAR_SIZE;
+      if (rest!=0) {
+	if (typ2!=aet_bit)
+	  goto badcopy;
+	while(rest> 0) {
+	  fSaset1(y,i2+n1-rest,(fLrow_major_aref(x,i1+n1-rest)));
+	  rest--;
+	}
+      }
+      i1=i1/CHAR_SIZE ;
+      n1=n1/CHAR_SIZE;
+      typ1=aet_char;
+    }
+  }
+
+  if (typ2==aet_bit) {
+    if (i2 % CHAR_SIZE)
+      goto badcopy;
+    i2=i2/CHAR_SIZE ;
+  }
+
+  if ((typ1 ==aet_object || typ2  ==aet_object) && typ1 != typ2)
     FEerror("Can't copy between different array types",0);
   nc=n1 * aet_sizes[(int)typ1];
-  if (i1+n1 > x->a.a_dim
-      || ((y->a.a_dim - i2) *aet_sizes[(int)typ2]) < nc)
+  if (i1+n1 > x->a.a_dim || ((y->a.a_dim - i2) *aet_sizes[(int)typ2]) < nc)
     FEerror("Copy  out of bounds",0);
   bcopy(x->ust.ust_self + (i1*aet_sizes[(int)typ1]),
 	y->ust.ust_self + (i2*aet_sizes[(int)typ2]),
 	nc);
+
   return x;
+
 }
 
 /* X is the header of an array.  This supplies the body which
@@ -963,7 +984,7 @@ array_allocself(object x, int staticp, object dflt)
 	default:
 	  break;
 	}
-	if(dflt!=0) gset(x->st.st_self,raw_aet_ptr(dflt,typ),n,typ);
+	if(dflt!=OBJNULL) gset(x->st.st_self,raw_aet_ptr(dflt,typ),n,typ);
       }
 	
 }
@@ -1122,9 +1143,9 @@ Icheck_displaced(object displaced_list, object ar, int dim)
 /*  } */
 /* } */
 
-DEFUNO_NEW("REPLACE-ARRAY",object,fSreplace_array,SI,2,2,NONE,
-       OO,OO,OO,OO,void,siLreplace_array,(object old,object new),"")
-{ struct dummy fw ;
+DEFUN_NEW("REPLACE-ARRAY",object,fSreplace_array,SI,2,2,NONE,OO,OO,OO,OO,(object old,object new),"") {
+
+  struct dummy fw;
   fw = old->d;
 
   old = IisArray(old);
@@ -1158,10 +1179,9 @@ DEFUNO_NEW("REPLACE-ARRAY",object,fSreplace_array,SI,2,2,NONE,
   return old;
 }
 
-DEFUN_NEW("ARRAY-TOTAL-SIZE",object,fLarray_total_size,LISP,1,1,
-       NONE,OO,OO,OO,OO,(object x),"")
+DEFUN_NEW("ARRAY-TOTAL-SIZE",object,fLarray_total_size,LISP,1,1,NONE,IO,OO,OO,OO,(object x),"")
 { x = IisArray(x);
-  return make_fixnum(x->a.a_dim);
+  return (object)(fixnum)x->a.a_dim;
 }
 
 
@@ -1491,15 +1511,15 @@ DEFUN_NEW("ASET-BY-CURSOR",object,fSaset_by_cursor,SI,3,3,
 				ind[43],ind[44],ind[45],ind[46],ind[47],ind[48],ind[49],
 				ind[50],ind[51],ind[52],ind[53],ind[54],ind[55],ind[56],
 				ind[57],ind[58],ind[59],ind[60],ind[61],ind[62]);break;
-	case 64:  (*FFN(fSaset))(ind[0],ind[1],ind[2],ind[3],ind[4],ind[5],ind[6],ind[7],
-				ind[8],ind[9],ind[10],ind[11],ind[12],ind[13],ind[14],
-				ind[15],ind[16],ind[17],ind[18],ind[19],ind[20],ind[21],
-				ind[22],ind[23],ind[24],ind[25],ind[26],ind[27],ind[28],
-				ind[29],ind[30],ind[31],ind[32],ind[33],ind[34],ind[35],
-				ind[36],ind[37],ind[38],ind[39],ind[40],ind[41],ind[42],
-				ind[43],ind[44],ind[45],ind[46],ind[47],ind[48],ind[49],
-				ind[50],ind[51],ind[52],ind[53],ind[54],ind[55],ind[56],
-				ind[57],ind[58],ind[59],ind[60],ind[61],ind[62],ind[63]);break;
+/* 	case 64:  (*FFN(fSaset))(ind[0],ind[1],ind[2],ind[3],ind[4],ind[5],ind[6],ind[7], */
+/* 				ind[8],ind[9],ind[10],ind[11],ind[12],ind[13],ind[14], */
+/* 				ind[15],ind[16],ind[17],ind[18],ind[19],ind[20],ind[21], */
+/* 				ind[22],ind[23],ind[24],ind[25],ind[26],ind[27],ind[28], */
+/* 				ind[29],ind[30],ind[31],ind[32],ind[33],ind[34],ind[35], */
+/* 				ind[36],ind[37],ind[38],ind[39],ind[40],ind[41],ind[42], */
+/* 				ind[43],ind[44],ind[45],ind[46],ind[47],ind[48],ind[49], */
+/* 				ind[50],ind[51],ind[52],ind[53],ind[54],ind[55],ind[56], */
+/* 				ind[57],ind[58],ind[59],ind[60],ind[61],ind[62],ind[63]);break; */
 	default: FEerror("Exceeded call-arguments-limit ",0);
 	} 
 	
