@@ -30,9 +30,11 @@ enum type what_to_collect;
 
 object sSAprofile_arrayA;
 #ifdef NO_PROFILE
-void
-profil(void)
-{;}
+#ifdef DARWIN/*FIXME macosx10.8 has a prototype (which must match here) but unlinkable function in 64bit*/
+int profil(char *buf, size_t bufsiz, unsigned long offset, unsigned int scale){return 0;}
+#else
+void profil(void){;}
+#endif
 #endif
 
 
@@ -44,28 +46,33 @@ DEFUN_NEW("PROFILE",object,fSprofile,SI
 {				/* 2 args */
   
   object ar=sSAprofile_arrayA->s.s_dbind;
+  void *x;
+  fixnum a,s;
+
   if (type_of(ar)!=t_string)
     FEerror("si:*Profile-array* not a string",0);
   if( type_of(start_address)!=t_fixnum ||   type_of(scale)!=t_fixnum)
     FEerror("Needs start address and scale as args",0);
 
-  profil((void *) (ar->ust.ust_self), (ar->ust.ust_dim),
-	 fix(start_address),fix(scale) << 8);
+  massert((a=fix(start_address))>=0);
+  massert((s=fix(scale))>=0);
+
+  x=a&&s ? (void *) (ar->ust.ust_self) : NULL;
+  profil(x, (ar->ust.ust_dim),fix(start_address),fix(scale) << 8);
   RETURN1(start_address);
 }
 
 #endif
-DEFUN_NEW("FUNCTION-START",object,fSfunction_start,SI
-       ,1,1,NONE,OO,OO,OO,OO,(object funobj),"")
-{/* 1 args */
- if(type_of(funobj)!=t_cfun
-    && type_of(funobj)!=t_sfun
-    && type_of(funobj)!=t_vfun
-    && type_of(funobj)!=t_afun
-    && type_of(funobj)!=t_gfun)
-    FEerror("not compiled function",0);
- funobj=make_fixnum((long) (funobj->cf.cf_self));
- RETURN1(funobj);
+DEFUN_NEW("FUNCTION-START",object,fSfunction_start,SI,1,1,NONE,OO,OO,OO,OO,(object funobj),"") {
+
+  switch (type_of(funobj)) {
+  case t_cfun:case t_sfun:case t_vfun:case t_afun:case t_gfun:case t_closure:case t_cclosure:
+    return make_fixnum((long) (funobj->cf.cf_self));
+  default:
+    TYPE_ERROR(funobj,sLcompiled_function);
+    return Cnil;
+  }
+
 }
 
 /* begin fasl stuff*/
@@ -77,82 +84,6 @@ DEFUN_NEW("FUNCTION-START",object,fSfunction_start,SI
 char *data_load_addr =0;
 #endif
 
-
-#ifdef SPECIAL_RSYM
-void
-read_special_symbols(symfile)
-char *symfile;
-{FILE *symin;
- char *symbols;
- int i,jj;
- struct lsymbol_table tab;
-#ifdef AIX3
- {char buf[500];
-  struct ld_info * ld;
- loadquery(L_GETINFO,buf,sizeof(buf));
-  ld = (struct ld_info *)buf;
-  data_load_addr = ld->ldinfo_dataorg ;}
-#endif  
- if (!(symin=fopen(symfile,"r")))
-   {perror(symfile);exit(1);};
- if(!fread((char *)&tab,sizeof(tab),1,symin))
-   FEerror("No header",0);
-  symbols=malloc(tab.tot_leng);
- c_table.alloc_length=( (PTABLE_EXTRA+ tab.n_symbols));
- (c_table.ptable) = (TABL *) malloc(sizeof(struct node) * c_table.alloc_length);
- if (!(c_table.ptable)) {perror("could not allocate"); exit(1);};
- i=0; c_table.length=tab.n_symbols;
- while(i < tab.n_symbols)
-   { fread((char *)&jj,sizeof(int),1,symin);
-#ifdef FIX_ADDRESS
-     FIX_ADDRESS(jj);
-#endif       
-     (SYM_ADDRESS(c_table,i))=jj;
-     SYM_STRING(c_table,i)=symbols;
- 
-     while((*(symbols++) =   getc(symin)))
-       {;}
-/*     dprintf( name %s ,  SYM_STRING(c_table,i));
-     dprintf( addr %d , jj);
-*/
-     i++;
-   }
-
- /*
-   for(i=0;i< 5;i++)
-   {printf("Symbol: %d %s %d \n",i,SYM_STRINGN(c_table,i),
-   SYM_ADDRESS(*ptable,i));}
-   */
- if (symin) fclose(symin);
-}
-
-int
-node_compare(const void *node1,const void *node2)
-{ return(strcmp( ((struct node *)node1)->string,
-	         ((struct node *)node2)->string));}
-
-
-
-DEFUNO_NEW("READ-EXTERNALS",object,fSread_externals,SI
-       ,1,1,NONE,OO,OO,OO,OO,void,siLread_externals,(object x0),"")
-{/* 1 args */
- {object x=x0;
-  unsigned int n;
-  char *str;
-  n=x->st.st_fillp;
- check_type_string(&x);
- str=malloc(n+1);
-  str[n]=0;
- (void) strncpy(str,x->st.st_self,n);
- read_special_symbols(str);
-  /* we sort them since these are used by the sfasl loader too */
- qsort((char*)(c_table.ptable),(int)(c_table.length),sizeof(struct node),node_compare);
-  free(str);}
- RETURN1(x0);
-}
-
-#endif /* special_rsym */
-
 #define CFUN_LIM 10000
 
 int maxpage;
@@ -164,7 +95,7 @@ static void
 cfuns_to_combined_table(unsigned int n) /* non zero n will ensure new table length */
                
 {int ii=0;  
- STATIC int i, j;
+ STATIC int j;
  STATIC object x;
  STATIC char *p,*cf_addr;
  STATIC struct typemanager *tm;
@@ -172,40 +103,43 @@ cfuns_to_combined_table(unsigned int n) /* non zero n will ensure new table leng
  if (n && combined_table.alloc_length < n)
    { 
      (combined_table.ptable)=NULL;
-     (combined_table.ptable)= (TABL *)malloc(n* sizeof(struct node));
+     (combined_table.ptable)= (struct node *)malloc(n* sizeof(struct node));
      if(!combined_table.ptable)
        FEerror("unable to allocate",0);
      combined_table.alloc_length=n;}
 
- for (i = 0;  i < maxpage;  i++) {
-   if ((enum type)type_map[i]!=tm_table[(short)t_cfun].tm_type &&
-       (enum type)type_map[i]!=tm_table[(short)t_gfun].tm_type &&
-       (enum type)type_map[i]!=tm_table[(short)t_sfun].tm_type &&
-       (enum type)type_map[i]!=tm_table[(short)t_vfun].tm_type
-       )
+ {
+   struct pageinfo *v;
+   for (v=cell_list_head;v;v=v->next) {
+     enum type tp=v->type;
+     if (tp!=tm_table[(short)t_cfun].tm_type &&
+	 tp!=tm_table[(short)t_gfun].tm_type &&
+	 tp!=tm_table[(short)t_sfun].tm_type &&
+	 tp!=tm_table[(short)t_vfun].tm_type
+	 )
      continue;
-   tm = tm_of((enum type)type_map[i]);
-   p = pagetochar(i);
-   for (j = tm->tm_nppage; j > 0; --j, p += tm->tm_size) {
-     x = (object)p;
-     if (type_of(x)!=t_cfun &&
-	 type_of(x)!=t_sfun &&
-	 type_of(x)!=t_vfun &&
-	 type_of(x)!=t_gfun
-	 ) continue;
-     if ((x->d.m == FREE) || x->cf.cf_self == NULL)
-       continue;
-	/* the cdefn things are the proclaimed call types. */
-     cf_addr=(char * ) ((unsigned long)(x->cf.cf_self));
-	
-     SYM_ADDRESS(combined_table,ii)=(unsigned long)cf_addr;
-     SYM_STRING(combined_table,ii)= (char *)(CF_FLAG | (unsigned long)x) ;
-/*       (x->cf.cf_name ? x->cf.cf_name->s.st_self : NULL) ; */
-     combined_table.length = ++ii;
-     if (ii >= combined_table.alloc_length)
-       FEerror("Need a larger combined_table",0);
+     tm = tm_of(tp);
+     p = pagetochar(page(v));
+     for (j = tm->tm_nppage; j > 0; --j, p += tm->tm_size) {
+       x = (object)p;
+       if (type_of(x)!=t_cfun &&
+	   type_of(x)!=t_sfun &&
+	   type_of(x)!=t_vfun &&
+	   type_of(x)!=t_gfun
+	   ) continue;
+       if (is_free(x) || x->cf.cf_self == NULL)
+	 continue;
+       /* the cdefn things are the proclaimed call types. */
+       cf_addr=(char * ) ((unsigned long)(x->cf.cf_self));
+       
+       SYM_ADDRESS(combined_table,ii)=(unsigned long)cf_addr;
+       SYM_STRING(combined_table,ii)= (char *)(CF_FLAG | (unsigned long)x) ;
+       /*       (x->cf.cf_name ? x->cf.cf_name->s.st_self : NULL) ; */
+       combined_table.length = ++ii;
+       if (ii >= combined_table.alloc_length)
+	 FEerror("Need a larger combined_table",0);
+     }
    }
-		
  }
 }
 
@@ -222,6 +156,8 @@ address_node_compare(const void *node1, const void *node2)
 
 #if defined(HAVE_LIBBFD) && ! defined(SPECIAL_RSYM)
 
+static int bfd_update;
+
 static MY_BFD_BOOLEAN
 bfd_combined_table_update(struct bfd_link_hash_entry *h,PTR ct) {
 
@@ -236,9 +172,14 @@ bfd_combined_table_update(struct bfd_link_hash_entry *h,PTR ct) {
     return MY_BFD_FALSE;
   }
 
-  SYM_ADDRESS(combined_table,combined_table.length)=h->u.def.value+h->u.def.section->vma;
-  SYM_STRING(combined_table,combined_table.length)=(char *)h->root.string;
-  
+  if (bfd_update) {
+    if (combined_table.length>=combined_table.alloc_length)
+      FEerror("combined table overflow", 0);
+    
+    SYM_ADDRESS(combined_table,combined_table.length)=h->u.def.value+h->u.def.section->vma;
+    SYM_STRING(combined_table,combined_table.length)=(char *)h->root.string;
+  }
+
   combined_table.length++;
 
   return MY_BFD_TRUE;
@@ -272,35 +213,42 @@ DEFUN_NEW("SET-UP-COMBINED",object,fSset_up_combined,SI
 
     int j,k;
 
-    if((k=combined_table.length)+c_table.length >=
-       combined_table.alloc_length)
-      cfuns_to_combined_table(combined_table.length+c_table.length +20);
-
+    if((k=combined_table.length)+c_table.length >=  combined_table.alloc_length)
+      cfuns_to_combined_table(combined_table.length+c_table.length+20);
+    
     for(j = 0; j < c_table.length;) { 
       SYM_ADDRESS(combined_table,k) =SYM_ADDRESS(c_table,j);
       SYM_STRING(combined_table,k) =SYM_STRING(c_table,j);
       k++;
       j++;
     }
-    combined_table.length += c_table.length ;}
+
+    combined_table.length += c_table.length ;
+
+  }
 
 #else
 #if defined(HAVE_LIBBFD)
   if (link_info.hash) {
 
-    if (combined_table.length+link_info.hash->table.size >=
-	combined_table.alloc_length)
-      cfuns_to_combined_table(combined_table.length+link_info.hash->table.size+20);
-
+    bfd_update=0;
     bfd_link_hash_traverse(link_info.hash,
-				 bfd_combined_table_update,&combined_table);
+			   bfd_combined_table_update,&combined_table);
+    
+    if (combined_table.length >=combined_table.alloc_length)
+      cfuns_to_combined_table(combined_table.length);
+    
+    bfd_update=1;
+    bfd_link_hash_traverse(link_info.hash,
+			   bfd_combined_table_update,&combined_table);
+    bfd_update=0;
 
   }
 #endif
 #endif
 
-  qsort((char*)combined_table.ptable,(int)combined_table.length,
-	sizeof(struct node),address_node_compare);
+  qsort(combined_table.ptable,combined_table.length,sizeof(*combined_table.ptable),address_node_compare);
+
   RETURN1(siz);
 
 }
@@ -326,11 +274,13 @@ endar=aar+dim;
 
 
 DEFUN_NEW("DISPLAY-PROFILE",object,fSdisplay_profile,SI
-       ,2,2,NONE,OO,OO,OO,OO,(object start_addr,object scal),"")
-{if (!combined_table.ptable)
-   FEerror("must symbols first",0);
-   /* 2 args */
-   {unsigned int prev,next,upto,dim,total;
+       ,2,2,NONE,OO,OO,OO,OO,(object start_addr,object scal),"") {
+
+  if (!combined_table.ptable)
+    FEerror("must symbols first",0);
+  /* 2 args */
+  {
+    unsigned int prev,next,upto,dim,total;
     int j,scale,count;
     unsigned char *ar;
     object obj_ar;
@@ -344,35 +294,42 @@ DEFUN_NEW("DISPLAY-PROFILE",object,fSdisplay_profile,SI
     dim= (obj_ar->ust.ust_dim);
 
     total=string_sum(ar,dim);
-  
+
     j=0;
-    {int i, finish = combined_table.length-1;
-     for(i =0,prev=SYM_ADDRESS(combined_table,i); i< finish;
-	 prev=next)
-       { ++i;
-	 next=SYM_ADDRESS(combined_table,i);
-	 if ( prev < prof_start) continue;
-	 upto=prof_ind(next,scale);
-	 if (upto >= dim) upto=dim;
-	 {char *name; unsigned long uname;
+    {
+      int i, finish = combined_table.length-1;
+      for(i =0,prev=SYM_ADDRESS(combined_table,i); i< finish;prev=next)	{
+	++i;
+	next=SYM_ADDRESS(combined_table,i);
+	if (prev<prof_start)
+	  continue;
+	upto=prof_ind(next,scale);
+	if (upto >= dim)
+	  upto=dim;
+	{
+	  const char *name; unsigned long uname;
 	  count=0;
-	  for( ; j<upto;j++)
+	  for(;j<upto;j++)
 	    count += ar[j];
 	  if (count > 0) {
 	    name=SYM_STRING(combined_table,i-1);
 	    uname = (unsigned long) name;
 	    printf("\n%6.2f%% (%5d): ",(100.0*count)/total, count);
 	    fflush(stdout);
-	    if (CF_FLAG & uname)
-	      {if (~CF_FLAG & uname) prin1( ((object) (~CF_FLAG & uname))->cf.cf_name,Cnil);}
-	     else if (name ) printf("%s",name);};
+	    if (CF_FLAG & uname) {
+	      if (~CF_FLAG & uname)
+		prin1( ((object) (~CF_FLAG & uname))->cf.cf_name,Cnil);
+	    } else if (name )
+	      printf("%s",name);};
 	  if (upto==dim) goto TOTALS ;
-	  
-	}}}
- TOTALS:
-  printf("\nTotal ticks %d",total);fflush(stdout);
+
+	}
+      }
+    }
+  TOTALS:
+    printf("\nTotal ticks %d",total);fflush(stdout);
   }
- RETURN1(start_addr);
+  RETURN1(start_addr);
 }
 
 
@@ -426,13 +383,13 @@ object x0,x1;
 
 DEFVAR("*PROFILE-ARRAY*",sSAprofile_arrayA,SI,Cnil,"");
 void
-init_fat_string(void)
+gcl_init_fat_string(void)
 {
  
  make_si_constant("*ASH->>*",(-1==(((int)-1) >> 20))? Ct :Cnil);
-#ifdef SFASL
- make_si_function("BUILD-SYMBOL-TABLE",build_symbol_table);
-#endif
+/* #ifdef SFASL */
+/*  make_si_function("BUILD-SYMBOL-TABLE",build_symbol_table); */
+/* #endif */
 
 
  init_fasdump();
