@@ -55,31 +55,43 @@ object small_fixnum ( int i ) {
 }
 #endif
 
+/* #if !defined(IM_FIX_BASE) */
+
 #define BIGGER_FIXNUM_RANGE
 
 #ifdef BIGGER_FIXNUM_RANGE
 struct {int min,max;} bigger_fixnums;
 
 struct fixnum_struct *bigger_fixnum_table;
-DEFUN_NEW("ALLOCATE-BIGGER-FIXNUM-RANGE",object,fSallocate_bigger_fixnum_range,
-      SI,2,2,NONE,OI,IO,OO,OO,(fixnum min,fixnum max),"") 
-{ int j; 
-  if (min <= max); else {FEerror("Need Min < Max",0);}
-  bigger_fixnum_table= (void *) malloc(sizeof(struct fixnum_struct)*
-				       (max - min));
+#if !defined(IM_FIX_BASE) || defined(USE_SAFE_CDR)
+#define STATIC_BIGGER_FIXNUM_TABLE_BITS 10
+static struct fixnum_struct bigger_fixnum_table1[1<<(STATIC_BIGGER_FIXNUM_TABLE_BITS+1)] OBJ_ALIGN;
+#endif
+
+DEFUN_NEW("ALLOCATE-BIGGER-FIXNUM-RANGE",object,fSallocate_bigger_fixnum_range,SI,2,2,NONE,OI,IO,OO,OO,(fixnum min,fixnum max),"")  {
+  int j; 
+  if (min > max) FEerror("Need Min <= Max",0);
+
+#if !defined(IM_FIX_BASE) || defined(USE_SAFE_CDR)
+  if (min==-(1<<STATIC_BIGGER_FIXNUM_TABLE_BITS) && max==(1<<STATIC_BIGGER_FIXNUM_TABLE_BITS))
+    bigger_fixnum_table=bigger_fixnum_table1;
+  else
+#endif
+    bigger_fixnum_table=(void *)malloc(sizeof(struct fixnum_struct)*(max - min));
   
-  for (j=min ; j < max ; j=j+1)
-    { 		bigger_fixnum_table[j - min].t
-		= (short)t_fixnum;
-		bigger_fixnum_table[j - min].FIXVAL = j;
-	      }
+  for (j=min ; j < max ; j=j+1) { 		
+    object x=(object)(bigger_fixnum_table+j-min);
+    x->fw=0;
+    set_type_of(x,t_fixnum);
+    x->FIX.FIXVAL=j;
+  }
   bigger_fixnums.min=min;
   bigger_fixnums.max=max;
   
   return Ct;
 }
 #endif
-
+/* #endif */
 
 
 object
@@ -99,14 +111,14 @@ make_fixnum1(long i)
 #endif	
 	      
 	x = alloc_object(t_fixnum);	    
-	fix(x) = i;
+	set_fix(x,i);
 	return(x);
 }
 
 object
 make_ratio(object num, object den)
 {
-	object g, r, integer_divide1(object x, object y), get_gcd(object x, object y);
+	object g, r, get_gcd(object x, object y);
 	vs_mark;
 
 	if (number_zerop(den))
@@ -123,9 +135,9 @@ make_ratio(object num, object den)
 	}
 	g = get_gcd(num, den);
 	vs_push(g);
-	num = integer_divide1(num, g);
+	num = integer_divide1(num, g,0);
 	vs_push(num);
-	den = integer_divide1(den, g);
+	den = integer_divide1(den, g,0);
 	vs_push(den);
 	if(type_of(den) == t_fixnum && fix(den) == 1) {
 		vs_reset;
@@ -245,6 +257,12 @@ number_to_double(object x)
 
 	case t_ratio:
 	  
+	  /* vs_base=vs_top; */
+	  /* vs_push(x); */
+	  /* Lround(); */
+	  /* if (vs_base[0]!=small_fixnum(0))  */
+	  /*   return number_to_double(vs_base[0])+number_to_double(vs_base[1]); */
+	  /* else */
 	  {
 	    double dx,dy;
 	    object xx,yy;
@@ -255,14 +273,14 @@ number_to_double(object x)
 	      if (ISNORMAL(dx))
 		dx*=0.5;
 	      else {
-		xx=integer_divide1(xx,small_fixnum(2));
+		xx=integer_divide1(xx,small_fixnum(2),0);
 		dx=number_to_double(xx);
 	      }
 
 	      if (ISNORMAL(dy))
 		dy*=0.5;
 	      else {
-		yy=integer_divide1(yy,small_fixnum(2));
+		yy=integer_divide1(yy,small_fixnum(2),0);
 		dy=number_to_double(yy);
 	      }
 
@@ -286,13 +304,10 @@ number_to_double(object x)
 void
 gcl_init_number(void)
 {
-	int i;
 
-	for (i = -SMALL_FIXNUM_LIMIT;  i < SMALL_FIXNUM_LIMIT;  i++) {
-		small_fixnum_table[i + SMALL_FIXNUM_LIMIT].t
-		= (short)t_fixnum;
-		small_fixnum_table[i + SMALL_FIXNUM_LIMIT].FIXVAL = i;
-	}
+#if !defined(IM_FIX_BASE) || defined(USE_SAFE_CDR)
+  FFN(fSallocate_bigger_fixnum_range)(-1024,1024);
+#endif
 
 	shortfloat_zero = alloc_object(t_shortfloat);
 	sf(shortfloat_zero) = (shortfloat)0.0;
