@@ -33,8 +33,8 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #  include <netinet/in.h>
 #  include <arpa/inet.h>
 #else
-#  include <windows.h>
 #  include <winsock2.h>
+#  include <windows.h>
 #endif
 
 #ifdef __STDC__
@@ -70,7 +70,7 @@ int w32_socket_init(void)
     } else {
         if (WSAStartup(0x0101, &WSAData)) {
             w32_socket_initialisations = 0;
-            fprintf ( stderr, "WSAStartup failed\n" );
+            emsg("WSAStartup failed\n" );
             WSACleanup();
             rv = -1;
         }
@@ -158,13 +158,9 @@ the socket.  If PORT is zero do automatic allocation of port")
 #endif                
                 (cRetry < BIND_MAX_RETRY));
       if (0)
-	{
-	  fprintf(stderr,
-  "\nAssigned automatic address to socket : port(%d), errno(%d), bind_rc(%d), iLastAddressUsed(%d), retries(%d)\n"
+	  emsg("\nAssigned automatic address to socket : port(%d), errno(%d), bind_rc(%d), iLastAddressUsed(%d), retries(%d)\n"
 		  , addr.sin_port, errno, rc, iLastAddressUsed, cRetry
 		  );
-	  fflush(stderr);
-	}
     }
   else
     {
@@ -213,15 +209,15 @@ DEFUN_NEW("ACCEPT-SOCKET-CONNECTION",object,fSaccept_socket_connection,
 and returns (list* named_socket fd name1) when one is established")
 
 {
-  int n, fd;
+  socklen_t n;
+  int fd;
   struct sockaddr_in addr;
   object x; 
   n = sizeof addr;
   fd = accept(fix(car(named_socket)) , (struct sockaddr *)&addr, &n);
   if (fd < 0)
     {
-      perror("ERROR ! accept on socket failed in sock_accept_connection");
-      fflush(stderr);
+      emsg("ERROR ! accept on socket failed in sock_accept_connection");
       return Cnil;
     }
   x = alloc_simple_string(sizeof(struct connection_state));
@@ -261,7 +257,12 @@ DEFUN_NEW("HOSTNAME-TO-HOSTID",object,fShostname_to_hostid,SI,1,1,
   char buf[300];
   char *p;
   p = lisp_copy_to_null_terminated(host,buf,sizeof(buf));
-  h = gethostbyname(p);
+  h = /* gethostbyname(p); */
+#ifdef STATIC_LINKING
+    NULL;
+#else
+  gethostbyname(p);
+#endif
   if (p != buf) free (p);
   if (h && h->h_addr_list[0])
     return
@@ -287,7 +288,12 @@ DEFUN_NEW("HOSTID-TO-HOSTNAME",object,fShostid_to_hostname,SI,
   char buf[300];
   hostid = lisp_copy_to_null_terminated(host_id,buf,sizeof(buf));
   addr.s_addr = inet_addr(hostid);
-  h = gethostbyaddr((char *)&addr, 4, AF_INET);
+  h = /* gethostbyaddr((char *)&addr, 4, AF_INET); */
+#ifdef STATIC_LINKING
+    NULL;
+#else
+  gethostbyaddr((char *)&addr, 4, AF_INET);
+#endif
   if (h && h->h_name && *h->h_name)
     return make_simple_string(h->h_name);
   else
@@ -320,14 +326,14 @@ DEFUN_NEW("CONNECTION-STATE-FD",object,fSconnection_state_fd,SI,1,1,NONE,OO,OO,O
      
 DEFUN_NEW("OUR-WRITE",object,fSour_write,SI,3,3,NONE,OO,OI,OO,OO,(object sfd,object buffer,fixnum nbytes),"")
 
-{ return make_fixnum(write1(OBJ_TO_CONNECTION_STATE(sfd),buffer->ust.ust_self,nbytes));
+{ return make_fixnum(write1(OBJ_TO_CONNECTION_STATE(sfd),buffer->st.st_self,nbytes));
 }
 
 DEFUN_NEW("OUR-READ-WITH-OFFSET",object,fSour_read_with_offset,SI,5,5,NONE,
 	  OO,OI,II,OO,(object fd,object buffer,fixnum offset,fixnum nbytes,fixnum timeout),
       "Read from STATE-FD into string BUFFER putting data at OFFSET and reading NBYTES, waiting for TIMEOUT before failing")
 
-{ return make_fixnum(read1(OBJ_TO_CONNECTION_STATE(fd),&((buffer)->ust.ust_self[offset]),nbytes,timeout));
+{ return make_fixnum(read1(OBJ_TO_CONNECTION_STATE(fd),&((buffer)->st.st_self[offset]),nbytes,timeout));
 }
 
 
@@ -421,7 +427,7 @@ fill pointer, and this will be advanced.")
 
 
     break;
-  default: abort();
+  default: do_gcl_abort();
   }
   
   switch (t) {
@@ -435,7 +441,7 @@ fill pointer, and this will be advanced.")
      if (downcase)
      while (--len>=0)
        { char c = *p++;
-	 c=tolower(c);
+	 c=tolower((int)c);
 	 if(needs_quoting[(unsigned char)c])
 	   PUSH('\\');
 	 PUSH(c);}
@@ -512,16 +518,6 @@ DEFUN_NEW("SET-SIGIO-FOR-FD",object,fSset_sigio_for_fd,SI,1,1,NONE,OI,OO,OO,OO,(
 
 }
      
-DEFUN_NEW("RESET-STRING-INPUT-STREAM",object,fSreset_string_input_stream,SI,4,4,NONE,OO,OI,IO,OO,(object strm,object string,fixnum start,fixnum end),
-      "Reuse a string output STREAM by setting its output to STRING \
-and positioning the ouput/input to start at START and end at END")
-
-{ strm->sm.sm_object0 = string;
-  strm->sm.sm_int0 = start;
-  strm->sm.sm_int1 = end;
-  return strm;
-}
-
 DEFUN_NEW("CHECK-STATE-INPUT",object,fScheck_state_input,SI,2,2,NONE,OO,IO,OO,OO,(object osfd,fixnum timeout),
       "") 
 {
